local _G = _G

-- addon name and namespace
local ADDON, NS = ...

local Addon = LibStub("AceAddon-3.0"):GetAddon(ADDON)

-- get translations
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale(ADDON)

-- lib containing map data
local LibMapData = LibStub:GetLibrary("HereBeDragons-1.0")

-- the module name
local MODULE      = "Worldmap"
local DESCRIPTION = L["Module that provides direction arrow on the Worldmap."]

-- local functions
local cos     = math.cos
local sin     = math.sin

local WorldMapButton = WorldMapButton

local GetCurrentMapAreaID = _G.GetCurrentMapAreaID
local GetCVar             = _G.GetCVar
local GetPlayerFacing     = _G.GetPlayerFacing

local DIST_FROM_CENTER = 40

local ARROW = "Interface\\Addons\\"..ADDON.."\\minimap-magnet.tga"

-- aux variables
local _

-- the module
local Outlet = NS:CreateOutlet(MODULE, DESCRIPTION)

-- the arrow frame
do
	Outlet.arrow = CreateFrame("Frame", ADDON .. "_" .. MODULE .. "_Worldmap_Arrow", WorldMapButton)
	Outlet.arrow:SetHeight(24)
	Outlet.arrow:SetWidth(24)
	Outlet.arrow:SetParent(WorldMapButton)
	Outlet.arrow:SetFrameStrata(WorldMapButton:GetFrameStrata())
	Outlet.arrow:SetFrameLevel(WorldMapButton:GetFrameLevel() + 7)

	local texture = Outlet.arrow:CreateTexture(ADDON .. "_" .. MODULE .. "_Worldmap_Arrow_Tex", "OVERLAY")
	Outlet.arrow.texture = texture
	texture:SetAllPoints(Outlet.arrow)
	texture:SetTexture(ARROW)
	texture:SetTexCoord(0, 1, 0, 1)
end

Outlet:SetReady(LibMapData and true)

-- register
NS:RegisterModule(Outlet)

-- implement functions for output
function Outlet:Refresh()
	if not self:IsOperational() then
		return
	end

	local magnitude, direction, sinDir, cosDir = Addon:GetForce()

	if magnitude == 0 then
		self:Clear()
	
		return
	end
	
	local mapID = GetCurrentMapAreaID()
	
	if LibMapData:GetCZFromMapID(mapID) <= 0 then 
		return 
	end
	
	local pX, pY = LibMapData:GetPlayerZonePosition()
	local width, height = WorldMapButton:GetWidth(), WorldMapButton:GetHeight()
	
	local x = (pX * width) + (-DIST_FROM_CENTER * sinDir)
	local y = ((1 - pY) * height) + (DIST_FROM_CENTER * cosDir)
	
	-- border checks
	if x < 12 then
		x = 12
	elseif x > width - 12 then
		x = width - 12
	end

	if y < 12 then
		y = 12
	elseif y > height - 12 then
		y = height - 12
	end
		
	-- determine rotated and scaled texture coordinates
	local gX = (-sinDir + cosDir) * 0.5
	local gY = (-sinDir - cosDir) * 0.5
	
	self.arrow:ClearAllPoints()
	
	self.arrow.texture:SetTexCoord(
		0.5 - gX, 0.5 + gY,
		0.5 + gY, 0.5 + gX,
		0.5 - gY, 0.5 - gX,
		0.5 + gX, 0.5 - gY)
				
	self.arrow:SetPoint("CENTER", WorldMapButton, "BOTTOMLEFT", x, y)
	
	self.arrow:Show()
end

function Outlet:Clear()
	if not self:IsReady() then
		return
	end

	if self.arrow:IsShown() then
		self.arrow:Hide()
	end
end

-- test
function Outlet:Debug(msg)
	Addon:Debug("(Outlet:" .. MODULE .. ") " .. msg)
end
